unit GLOBALS; {Unit 1 of CONCRETE.PAS}

interface

  const
    TAB = 9;
    SPACE = 32;
    maxArray = 500;
    discrete = 1;
    continuous = 2;
    partial = 1;
    ordinary = 2;

  type
    complex = record
        re, im: real;
      end;

  type
    polar = record
        argu, modu: real;
      end;

  type
    points = record
        x, y: real
      end;

  type
    vector = array[0..50] of real;

  type
    shortVector = array[0..20] of real;

  type
    longVector = array[0..515] of real;

  type
    ivector = array[0..20] of integer;

  type
    pointsVector = array[0..515] of points;

  type
    complexVector = array[0..20] of complex;

  type
    polarVector = array[0..20] of polar;

  type
    matrix = array[0..20, 0..20] of real;

  type
    sPoint = array[0..10] of real; {sPoint}

  type
    sMatrix = array[0..10] of sPoint;

  type
    stringVector = array[1..4] of string;

  type
    IBOX = record
        xScale: boolean;
        yScale: boolean;
        xAxis: boolean;
        yAxis: boolean;
        hBorders: boolean;
        doFrame: boolean;
        doMarks: boolean;
        lollipops: boolean;
        X0: integer; {The horizontal value of the Graphics Origin}
        Y0: integer; {The vertical Value of the Graphics Origin}
        X1: integer; {Left of the Graphics Bounding Box}
        Y1: integer; {Top or Bottom  of the Graphics Bounding Box}
        X2: integer; {Right of the  GraphicsBounding Box}
        Y2: integer; {Bottom or Top of Graphics the Bounding Box}
        XC: integer; {horizontal value of the centre of window}
        YC: integer; {vertical value of the centre of window}
        BOXWDT: integer; {Width of the Bounding Box}
        BOXHGT: integer; {Height of the Bounding Box}
      end;

  type
    rBOX = record
        noData: boolean;
        empiricData: boolean;
        simData: boolean;
        logsTaken: boolean;
        diffOrder: integer;
        dataFrequency: integer; {Annual, Quarterly, Monthly, Other}
        NOPUNCT: integer; {number of data points}
        dataName: string;
        xMax: real; {the maximum value of x in the dataVector}
        xMin: real; {the minmum value  of x in the dataVector}
        yMax: real; {the maximum value of y in the dataVector}
        yMin: real; {the minimum value of y in the dataVector}
        yMean: real; {the mean value of y in the dataVector}
        yVar: real; {the variance of y in the dataVector}
        yRange: real; {yMax - yMin}
        xRange: real; {xMax - xMin;}
        yUB: real; {yUB >= yMax: an upper bound for y}
        yLB: real; {yLB <= yMin: a lower bound for y}
        xLB: real; {xLB <= xMin: a left (lower) bound for x}
        xRB: real; {xRB >= xMax: a right (upper) bound for x}
      end;

  type
    ARMARecord = record
        noModel: boolean;
        ordersSpecified: boolean;
        parametersFound: boolean;
        rootsFound: boolean;
        aCovarsFound: boolean;
        varEpsilon: real; {variance of white-noise forcing function}
        p: integer; {degree of the denominator}
        pr: integer; {no of pairs of complex denominator roots}
        q: integer; {degree of the numerator}
        qr: integer; {no pairs of complex numerator roots}
        alpha: shortVector; {coefficents of the denominator}
        beta: shortVector; {coefficents of the numerator}
        poles: complexVector; {Cartesian denominator roots}
        zeros: complexVector; {Cartesian numerator roots}
        polarPoles: polarVector; {polar denominator roots}
        polarZeros: polarVector; {polar numerator roots}
        aCovar: shortVector; {autocovariances}
      end;

{in the case of an discrete-time ARMA model the poles and zeros are the}
{inverse roots of the lag polynomial, which must lie inside the unit circle}

 {Graphics Prameters}
  var
    SCREENBOX, POSTBOX: IBOX;
    graphicsOn, noSCREENBOX: boolean;
    currentMode: integer;
    figNumber: integer;

  var
    discontent: integer; {1 => discrete, 2 => continuous}
    ARMASpec, CARMASpec, LSDESpec, EDLMSpec: ARMARecord;
    allsDone, PSPict, TeXFormat, noModelParams, noPageParams: boolean;
    dataBOX: rBOX;
{autoBOX: rBOX;}
    dataVector: longVector;
    clockStart: longint;

  procedure InitialiseARMARecord (var theRecord: ARMARecord);

  procedure InitialiseDataBOX (var dataBOX: rBOX);

  procedure InitialiseAnIBOX (var GraphBOX: IBOX);

implementation

{________________________________________________}

  procedure InitialiseARMARecord (var theRecord: ARMARecord);

    var
      j: integer;

  begin

    with theRecord do
      begin
        noModel := true;
        ordersSpecified := false;
        parametersFound := false;
        rootsFound := false;
        aCovarsFound := false;
        varEpsilon := 1.0; {variance of white-noise forcing function}
        p := 0; {degree of the denominator}
        pr := 0; {no of complex denominator roots}
        q := 0; {degree of the numerator}
        qr := 0; {no of complex numerator roots}
      end; {with theRecord}

  end; {InitialiseARMARecord}

{_________________________________________________}

  procedure InitialiseDataBOX (var dataBOX: rBOX);

  begin

    with dataBOX do
      begin
        noData := true;
        simData := false;
        empiricData := false;
        logsTaken := false;
        diffOrder := 0;
        dataName := 'an unnamed series';
      end;

  end; {InitialiseDataBOX}

{_________________________________________________}

  procedure InitialiseAnIBOX (var graphBOX: IBOX);

  begin
    with graphBOX do
      begin
        xScale := true;
        yScale := true;
        xAxis := true;
        yAxis := true;
        hBorders := true;
        doFrame := true;
        doMarks := true;
        lollipops := false;
      end;
  end; {InitialiseAnIBOX}

{_________________________________________________}

end. {GLOBALS: Unit 1 of CONCRETE.PAS}

