program MAIN; {Main Unit  of CONCRETE.PAS}

  uses
    GLOBALS, UTILS, ACQUIRE, CONVERTS, ORGANISE, Dos, Crt, Graph;

  var
    theMenuItem: integer;

begin
  SetTheText;

  Writeln;
  Writeln('    CONCRETE.PAS: Discrete and Continuous Linear Dynamic Models');
  Writeln;
  Writeln('    This program investigates  the relationship between dynamic linear');
  Writeln('    stochastic  models in discrete  and  continuous time. The discrete');
  Writeln('    time ARMA model is driven by a white-noise process that is limited');
  Writeln('    in  frequency  by the  Nyquist  value  of  pi radians  per  sample');
  Writeln('    interval. Its direct continuous time counterpart is  a CARMA model');
  Writeln('    driven  by  a  continuous frequency-limited  white-noise  process.');
  Writeln;
  Writeln('    The ARMA model may also represent  the  exact discrete counterpart');
  Writeln('    of a linear stochastic  differential  equation (LSDE) driven by an');
  Writeln('    unlimited  continuous  time  white-noise  process  comprising  the');
  Writeln('    infinitessimal increments of a Wiener process.');
  Writeln;
  Writeln('    The  program asks  the user to  specify  either an ARMA  model in');
  Writeln('    discrete time or a continuous  time LSDE. In the case  of an ARMA');
  Writeln('    specification, the  program  will derive  both  the corresponding');
  Writeln('    CARMA model and the LSDE model together with their autocovariance');
  Writeln('    functions and their spectral density functions. In the case of an');
  Writeln('    LSDE  specification, the program  will derive its  exact discrete');
  Writeln('    counterpart {EDLM).');
  Writeln(' ' : 61, '<RETURN>');
  Writeln;
  Readln;

  ClrScr;

  Writeln;
  Writeln('     GLOSSARY: The Discrete and Continuous Models and their Acronyms');

  Writeln;
  Writeln('     The Discrete-Time Autoregressive Moving Average Model:');

  Writeln;
  Writeln('         (alpha[0] + alpha[1]L + . . . + alpha[p]L^p)y(t)');
  Writeln('           = (beta[0] + beta[1]L + . . . + beta[q]L^q)epsilon(t)');

  Writeln;
  Writeln('     The Linear Stochastic Differential Equation:');

  Writeln;
  Writeln('         (phi[0]D^p + phi[1]D^{p-1} + . . . + phi[p])y(t)');
  Writeln('           = (theta[0]D^q + theta[1]D^{q-1} + . . . + theta[0])zeta(t)');
  Writeln;

  Writeln('     Acronyms');
  Writeln;
  Writeln('         ARMA: Autoregressive Moving Average');
  Writeln('         LSDE: Linear Stochastic Differential Equation');
  Writeln('         CARMA: Continuous-Time Autoregressive Moving Average');
  Writeln('         EDLM: Exact Discrete Linear Model');

  Writeln;
  Writeln('     N.B.');

  Writeln;
  Writeln('         The CARMA models are frequency-limited linear stochastic  ');
  Writeln('         differential equations corresponding to the ARMA models.');
  Writeln;
  Writeln('         The EDLM models are the discrete-time models that are ');
  Writeln('         the exact counterparts of the LSDE models. ');

  Writeln(' ' : 62, '<RETURN>');
  Readln;

  InitialiseARMARecord(ARMASpec);
  InitialiseARMARecord(LSDESpec);
  InitialiseARMARecord(CARMASpec);
  InitialiseARMARecord(EDLMSpec);
  InitialiseDataBoX(dataBOX);
  InitialiseAnIBOX(SCREENBOX);
  InitialiseAnIBOX(POSTBOX);

  discontent := 99;
  figNumber := 1;
  allsDone := false;
  noPageParams := true;
  noModelParams := true;
  graphicsOn := false;
  noSCREENBOX := true;
  SCREENBOX.XC := 0;
  SCREENBOX.YC := 0;

  repeat
    ClrScr;

    Writeln;
    Writeln('       DISCRETE AND CONTINUOUS ARMA MODELS');
    Writeln;
    Writeln('       1.  Get Page Parameters');
    Writeln;
    Writeln('       2.  Get Model Parameters');
    Writeln;
    Writeln('       3.  Find the Equivalent Discrete and Continuous Models');
    Writeln;
    Writeln('       4.  Plot the Autocovariances');
    Writeln;
    Writeln('       5.  Plot the Spectral Density Function');
    Writeln;
    Writeln('       6.  Assign Primary Status to a Derived Model');
    Writeln;
    Writeln('       7.  SAVE and EXIT');

    Writeln;
    Writeln('                      Select a number and press RETURN');

    GetBoundedInteger(1, 7, theMenuItem);

    case theMenuItem of
      1: 
        PageSetUp(SCREENBOX, POSTBOX, PSPict, TexFormat); {in UTILS}
      2: 
        GetModelParams(SCREENBOX, POSTBOX, ARMASpec, LSDESpec); {in ORGANISE}
      3: 
        DiscreteAndContinuousSpecs(SCREENBOX, POSTBOX, ARMASpec, cARMASpec, LSDESpec);
      4: 
        PlotAutoCovariances(SCREENBOX, POSTBOX, ARMASpec, LSDESpec); {in ORGANISE}
      5: 
        MakeTheSpectra(SCREENBOX, POSTBOX, ARMASpec, LSDESpec);
      6: 
        TransferStatus(ARMASpec, CARMASpec, LSDESpec, EDLMSpec);
      7: 
        SaveAndExit(ARMASpec, CARMASpec, LSDESpec, EDLMSpec); {in ORGANISE}
      otherwise
        ; {do nothing}
    end;{case}
  until allsDone;

  RevertText;

end. {MAIN: Program Unit 13 of CONCRETE.PAS}

 {________________________________________________}

   {Compile the Units in the Following Order:}

   {1. GLOBALS}
   {2. MATHS}
   {3. UTILS}
   {4. LEGENDS}
   {5. SCREEN}
   {6. POST}
   {9. SPECTRUM}
   {8. CACOVARS}
   {9. DACOVARS}
   {10. ROOTS}
   {11. OPTIMUM}
   {12. CONVERTS}
   {13. ACQUIRE}
   {14. ORGANISE}
   {15. MAIN}

