unit MATHS; {Unit 2 of CONTEXT.PAS}

interface

  uses
    GLOBALS;

  function Even (q: integer): boolean;

  function Min (p, q: integer): integer;

  function Max (p, q: integer): integer;

  function Rmin (x, y: real): real;

  function Rmax (x, y: real): real;

  function Cmod (a: complex): real;

  function CmodSqr (a: complex): real;

  procedure ComplexAdd (a, b: complex;
                  var result: complex);

  procedure ComplexSub (a, b: complex;
                  var result: complex);

  procedure ComplexMult (a, b: complex;
                  var result: complex);

  procedure ComplexInv (a: complex;
                 var result: complex);

  procedure ComplexDiv (a, b: complex;
                  var result: complex);

  function Sign (alpha: real): integer;

  function Csqrt (a: complex): complex;

  procedure ComplexSqrt (a: complex;
                  var result: complex);

  function TheArgument (thepoint: complex): real;

  function TheModulus (apoint: complex): real;

  procedure ComplexToPolar (complexPoint: complex;
                  var polarPoint: polar);

  procedure PolarToComplex (polarPoint: polar;
                  var complexPoint: complex);

  procedure ComplexLog (var polarPoint: polar;
                  var complexPoint: complex);

  procedure ComplexAntilog (var cNumber: complex;
                  var cAntilog: complex);

  function Power (x: real;
                  n: integer): real;

  function FractionalPower (x: real;
                  n: real): real;

  function RecursivePower (x: real;
                  n: integer): extended;

  procedure LUsolve (start, n: integer;
                  var a: matrix;
                  var x: shortVector;
                  b: shortVector);

  procedure RationalExpansion (var pulseBOX: rBOX;
                  var alpha, beta: vector;
                  p, k, n: integer);

implementation

{_________________________________________________}

  function Even (q: integer): boolean;

  begin
    Even := false;
    if q mod 2 = 0 then
      Even := true;
  end; {Even}

{_________________________________________________}

  function Min (p, q: integer): integer;
  begin
    if p < q then
      Min := p
    else
      Min := q
  end; {Min}

{_________________________________________________}

  function Max (p, q: integer): integer;
  begin
    if p > q then
      Max := p
    else
      Max := q
  end; {Max}

{_________________________________________________}

  function Rmin (x, y: real): real;
  begin
    if x < y then
      Rmin := x
    else
      Rmin := y
  end; {Rmin}

{_________________________________________________}

  function Rmax (x, y: real): real;
  begin
    if x > y then
      Rmax := x
    else
      Rmax := y
  end; {Rmax}

{_________________________________________________}

  function Cmod (a: complex): real;
  begin
    Cmod := Sqrt(Sqr(a.re) + Sqr(a.im));
  end; {Cmod: ComplexModulus}

{_________________________________________________}

  function CmodSqr (a: complex): real;
  begin
    CmodSqr := Sqr(a.re) + Sqr(a.im);
  end; {CmodSqr: Complex Squared Modulus}

{_________________________________________________}

  procedure ComplexAdd (a, b: complex;
                  var result: complex);
    var
      c: complex;
  begin
    c.re := a.re + b.re;
    c.im := a.im + b.im;
    result := c;
  end; {ComplexAdd: Complex Addition}

{_________________________________________________}

  procedure ComplexSub (a, b: complex;
                  var result: complex);
    var
      c: complex;
  begin
    c.re := a.re - b.re;
    c.im := a.im - b.im;
    result := c;
  end; {ComplexSub: Complex Subtraction }

{_________________________________________________}

  procedure ComplexMult (a, b: complex;
                  var result: complex);
    var
      c: complex;
  begin
    c.re := a.re * b.re - a.im * b.im;
    c.im := a.im * b.re + b.im * a.re;
    result := c;
  end; {Complex Multiplication}

{_________________________________________________}

 procedure ComplexInv (a: complex;
                 var result: complex);
   var
     c: complex;
 begin
   c.re := a.re / (Sqr(a.re) + Sqr(a.im));
   c.im := -a.im / (Sqr(a.re) + Sqr(a.im));
   result := c;
 end; {Complex Inversion}

{_________________________________________________}


  procedure ComplexDiv (a, b: complex;
                  var result: complex);
    var
      c: complex;
  begin
    c.re := (a.re * b.re + a.im * b.im) / (Sqr(b.re) + Sqr(b.im));
    c.im := (a.im * b.re - b.im * a.re) / (Sqr(b.re) + Sqr(b.im));
    result := c;
  end; {Complex Division}

{_________________________________________________}

  function Sign (alpha: real): integer;
  begin
    Sign := 0;
    if alpha < 0 then
      Sign := -1
    else
      Sign := 1;
  end; {Sign}

{_________________________________________________}

  function Csqrt (a: complex): complex;
    const
      virtualZero = 1E-10;
      pi = 3.1415926;
    var
      rho, theta: real;
      c: complex;

  begin {complex square root}
    rho := Sqrt(Sqr(a.re) + Sqr(a.im));

    if Abs(a.re) < virtualZero then
      theta := Sign(a.im) * pi / 2
    else
      theta := ArcTan(a.im / a.re);
    if a.re < 0 then
      theta := theta + Sign(a.im) * pi;

    c.re := Sqrt(rho) * Cos(theta / 2);
    c.im := Sqrt(rho) * Sin(theta / 2);
    Csqrt := c;

  end; {Csqrt: ComplexSquareRoot}

{_________________________________________________}

  procedure ComplexSqrt (a: complex;
                  var result: complex);

    var
      b: complex;
      rho: real;

  begin
    rho := Sqrt(a.re * a.re + a.im * a.im);
    b.re := Sqrt((rho + a.re) / 2);
    b.im := Sign(a.im) * Sqrt((rho - a.re) / 2);
    result := b;
  end; {ComplexSqrt}

{_________________________________________________}

  function TheArgument (thepoint: complex): real;

    var
      theta: real;

  begin
    if thepoint.re = 0 then
      theta := Sign(thepoint.im) * pi / 2;
    if thepoint.re <> 0 then
      theta := Arctan(thepoint.im / thepoint.re);
    if thepoint.re < 0 then
      theta := theta + Sign(thepoint.im) * pi;
    TheArgument := theta;
  end; {TheArgument}

{_________________________________________________}

  function TheModulus (apoint: complex): real;
  begin
    TheModulus := Sqrt(apoint.re * apoint.re + apoint.im * apoint.im);
  end; {TheModulus}

  {_________________________________________________}

  procedure ComplexToPolar (complexPoint: complex;
                  var polarPoint: polar);
  begin
    polarPoint.modu := TheModulus(complexPoint);
    polarPoint.argu := TheArgument(complexPoint);
  end; {ComplexToPolar}

  {_________________________________________________}

  procedure PolarToComplex (polarPoint: polar;
                  var complexPoint: complex);
  begin
    complexPoint.re := polarPoint.modu * Cos(polarPoint.argu);
    complexPoint.im := polarPoint.modu * Sin(polarPoint.argu);
  end; {PolarToComplex}

{_________________________________________________}

  procedure ComplexLog (var polarPoint: polar;
                  var complexPoint: complex);

{This take the logarithm of a complex exponential,}
{which  becomes a Cartesian complex number}

  begin
    complexPoint.re := Ln(polarPoint.modu);
    complexPoint.im := polarPoint.argu;
  end; {ComplexRoot}

{_________________________________________________}

  procedure ComplexAntilog (var cNumber: complex;
                  var cAntilog: complex);
    var
      modulus, argument: real;

{This takes the argument a  complex root of a kappa = ln(rho) + i*omega and}
{it delivers (rho, omega)  before calulating a + i*b with a = rho*cos (omega)}
{and b = rho*cos (omega)}

  begin
    modulus := Exp(cNumber.re);
    argument := cNumber.im;
    cAntilog.re := modulus * Cos(argument);
    cAntilog.im := modulus * Sin(argument);
  end; {ComplexAntilog}

{_________________________________________________}


  function Power (x: real;
                  n: integer): real;
    var
      logPower: real;
  begin
    x := Abs(x);
    if x < 1E-35 then
      Power := 0
    else
      begin {else}
        logPower := n * Ln(x);
        if (logPower < -80) then
          Power := 0
        else if (logPower > 80) then
          Power := Exp(80)
        else
          Power := Exp(logPower);
      end; {else}
  end; {Power}

{_________________________________________________}

  function FractionalPower (x: real;
                  n: real): real;
    var
      logPower: real;
  begin
    x := Abs(x);
    if x < 1E-35 then
      FractionalPower := 0
    else
      begin {else}
        logPower := n * Ln(x);
        if (logPower < -80) then
          FractionalPower := 0
        else if (logPower > 80) then
          FractionalPower := Exp(80)
        else
          FractionalPower := Exp(logPower);
      end; {else}
  end; {FractionalPower}

{_________________________________________________}

  function RecursivePower (x: real;
                  n: integer): extended;
  begin
    if n = 0 then
      RecursivePower := 1
    else
      RecursivePower := x * RecursivePower(x, n - 1);
  end; {RecursivePower}

{_________________________________________________}

  procedure LUsolve (start, n: integer;
                  var a: matrix;
                  var x: shortVector;
                  b: shortVector);

    var
      v, w, pivot, lambda: real;
      i, j, k, pivotRow, finish: integer;
      p: ivector;
      d: shortVector;

  begin {LUsolve}

    finish := start + n - 1;
    for i := start to finish do
      begin {i; determine the scale factors}
        p[i] := i;
        d[i] := 0.0;
        for j := start to finish do
          begin {j}
            if d[i] < Abs(a[i, j]) then
              d[i] := Abs(a[i, j]);
          end; {j}
      end; {i}

    for i := start to finish - 1 do
      begin {i; begin the process of reduction}
        pivot := a[p[i], i];
        for k := i + 1 to finish do
          begin {k; search for a better pivot}
            v := Abs(pivot) / d[p[i]];
            w := Abs(a[p[k], i]) / d[p[k]];
            if v < w then
              begin  {interchange rows if a better pivot is found}
                pivot := a[p[k], i];
                pivotRow := p[k];
                p[k] := p[i];
                p[i] := pivotRow;
              end; { end interchange}
          end; {k; end the search for a pivot}

        for k := i + 1 to finish do
          begin {k; eliminate a[k, i]}
            lambda := a[p[k], i] / pivot;
            for j := i + 1 to finish do
              a[p[k], j] := a[p[k], j] - lambda * a[p[i], j];
            a[p[k], i] := lambda   {save the multiplier }
          end; {k}
      end; {i; reduction completed}

    for i := start to finish do
      begin {i; forward substitution}
        x[i] := b[p[i]];
        for j := i - 1 downto start do
          x[i] := x[i] - a[p[i], j] * x[j];
      end; {i; forward substitution}

    for i := finish downto start do
      begin {i; back substitution}
        for j := i + 1 to finish do
          x[i] := x[i] - a[p[i], j] * x[j];
        x[i] := x[i] / a[p[i], i];
      end; {i; back substitution}

  end; {LUsolve}

{_________________________________________________}

  procedure RationalExpansion (var pulseBOX: rBOX;
                  var alpha, beta: vector;
                  p, k, n: integer);
    var
      i, j, r: integer;

  begin
    pulseBOX.yMax := 0.0;
    pulseBOX.yMin := 0.0;

    for j := 0 to n do
      begin {j}
        r := Min(p, j);
        if j > k then
          beta[j] := 0.0;
        for i := 1 to r do
          beta[j] := beta[j] - alpha[i] * beta[j - i];
        beta[j] := beta[j] / alpha[0];
        pulseBOX.yMax := Rmax(pulseBOX.yMax, beta[j]);
        pulseBOX.yMin := Rmin(pulseBOX.yMin, beta[j]);
      end; {j}

  end; {RationalExpansion}

{_________________________________________________}

end. {MATHS: Unit 2 of CONTEXT.PAS}

