unit POST; {Unit 6 of CONCRETE.PAS}

interface

  uses
    GLOBALS, UTILS, MATHS;

  procedure BlackDotDef (var F: text);

  procedure PostHeader (var F: text;
                  var POSTBOX: IBOX;
                  TeXFormat: boolean);

  procedure PostFooter (var F: text;
                  var POSTBOX: IBOX;
                  legend: string;
                  TexFormat: boolean);

  procedure PostMarks (var F: text;
                  XSTART, YSTART: real;
                  taly, firstMark, farBound, STEP: real;
                  Axis: string);

  procedure PostFrame (var F: text;
                  var dataBOX: rBOX;
                  var POSTBOX: IBOX);

  procedure PostVertAxis (var F: text;
                  XSTART, YSTART: integer;
                  taly, firstMark, farBound, VSTEP: real);

  procedure PostZeroPiAxis (var F: text;
                  var POSTBOX: IBOX);

  procedure PostPolyPiAxis (var F: text;
                  var POSTBOX: IBOX;
                  var spectraBOX: rBOX);

  procedure PostFrameLite (var F: text;
                  var realBOX: rBOX;
                  var POSTBOX: IBOX);

  procedure AutoPostFrame (var F: text;
                  lags: integer;
                  var POSTBOX: IBOX;
                  autoMin: real;
                  var autoRange: real);

  procedure PostSpikes (var F: text;
                  lags: integer;
                  var POSTBOX: IBOX;
                  var rho: shortVector;
                  autoRange: real);

  procedure PostPlot (var F: text;
                  var realBOX: rBOX;
                  var POSTBOX: IBOX;
                  var dataVector: longVector);

  procedure PostPlot2 (var F: text;
                  var realBOX: rBOX;
                  var POSTBOX: IBOX;
                  var dataVector: longVector);

  procedure PostBiPlot (var F: text;
                  var realBOX: rBOX;
                  var POSTBOX: IBOX;
                  var dataVector: pointsVector);

  procedure PostPlotMod (var F: text;
                  var realBOX: rBOX;
                  var POSTBOX: IBOX;
                  var dataVector: longVector);

  procedure WritePostFile (var dataBOX: rBOX;
                  var POSTBOX: IBOX;
                  var dataVector: longVector;
                  var legend: string);

implementation

{________________________________________________}

  procedure BlackDotDef (var F: text);

  begin
    Writeln(F, '/blackdot');
    Writeln(F, '{/radius exch def');
    Writeln(F, '/y exch def');
    Writeln(F, '/x exch def');
    Writeln(F, ' x  y  radius  0  360 arc fill');
    Writeln(F, '} def');
  end; {BlackDotDef}

{________________________________________________}

  procedure PostHeader (var F: text;
                  var POSTBOX: IBOX;
                  TeXFormat: boolean);
    var
      BOXHGT, BOXWDT, INDENT: integer;
      LLx, LLy, URx, URy: integer;
      xOrigin, yOrigin: integer;

  begin

    BOXWDT := POSTBOX.X2 - POSTBOX.X1;
    BOXHGT := POSTBOX.Y2 - POSTBOX.Y1;

    xOrigin := (595 - BOXWDT) div 2;
    yOrigin := 730 - BOXHGT;

    if TeXFormat then
      Writeln(F, '\special{postscript')
    else
      begin
        LLx := xOrigin - 40;
        LLy := yOrigin - 25;
        URx := xOrigin + BOXWDT + 40;
        URy := yOrigin + BOXHGT + 10;
        Writeln(F, '%!PS-Adobe-2.0');
        Writeln(F, '%%Creator: SigmaPi');
        Writeln(F, '%%BoundingBox:', LLx : 3, '  ', LLy : 3, '  ', URx : 3, '  ', URy : 3);
      end;

    Writeln(F, '/showxhatch');
    Writeln(F, '{/label exch def');
    Writeln(F, '/y exch def');
    Writeln(F, '/x exch def');
    Writeln(F, 'x  y  moveto');
    Writeln(F, '0  5 rlineto');
    Writeln(F, 'stroke');
    Writeln(F, 'x  y  moveto');
    Writeln(F, '0 -20 rmoveto');
    Writeln(F, 'label dup stringwidth pop 2 div neg 0 rmoveto show} def');

    Writeln(F, '/showyhatch');
    Writeln(F, '{/label exch def');
    Writeln(F, '/y exch def');
    Writeln(F, '/x exch def');
    Writeln(F, 'x  y  moveto');
    Writeln(F, ' 5 0 rlineto');
    Writeln(F, 'stroke');
    Writeln(F, 'x  y  moveto');
    Writeln(F, '-15 -2 rmoveto');
    Writeln(F, 'label dup stringwidth pop neg 0 rmoveto show} def');

    Writeln(F, '/showqhatch');
    Writeln(F, '{/label exch def');
    Writeln(F, '/y exch def');
    Writeln(F, '/x exch def');
    Writeln(F, 'x  y moveto');
    Writeln(F, ' -5 0 rlineto');
    Writeln(F, 'stroke');
    Writeln(F, 'x  y  moveto');
    Writeln(F, '30 -2 rmoveto');
    Writeln(F, 'label dup stringwidth pop neg 0 rmoveto show} def');

    if TeXFormat then
      begin {TeXFormat}
        INDENT := Round(((12 * 72) / 2.54 - BOXWDT) / 2);
        Writeln(F, INDENT : 2, '  ', -(BOXHGT + 15) : 2, '  translate');
      end {TeXFormat}
    else
      Writeln(F, xOrigin : 3, '  ', yOrigin : 3, ' translate ');

    Writeln(F, 'newpath');
    Writeln(F, '0.5 setlinewidth');

  end; {PostHeader}

   {_________________________________________________}

  procedure PostFooter (var F: text;
                  var POSTBOX: IBOX;
                  legend: string;
                  TexFormat: boolean);

    var
      BOXHGT: integer;

  begin

    if TexFormat then
      begin
        Writeln(F, 'showpage}');
        BOXHGT := POSTBOX.Y2 - POSTBOX.Y1;
        Writeln(F, '\vbox to ', BOXHGT + 35 : 3, ' pt');
        Writeln(F, ' {\hrule width 12cm height 0cm depth 0cm \vfill}');
        Writeln(F, '\medskip');
        Writeln(F, '{\par\noindent{\bf Figure ', figNumber : 1, '.} ');
        Write(F, legend);
        Writeln(F, '\par}');
        Writeln(F, '\medskip');
      end
    else
      begin
        Writeln(F, 'showpage');
        Writeln(F, '%%Trailer');
        Writeln(F, '%%EOF');
        Writeln(F, '%% Figure ', fignumber : 1, '. ', legend);
      end;

    figNumber := figNumber + 1;

  end; {PostFooter}

   {_________________________________________________}

  procedure PostMarks (var F: text;
                  XSTART, YSTART: real;
                  taly, firstMark, farBound, STEP: real;
                  Axis: string);

    var
      toly, HR, VT: real;
      tolyString: string;

  begin {PostMarks}

    toly := firstMark;
    HR := XSTART;
    VT := YSTART;

    while Abs(toly) <= Abs(farBound) do

      begin {while toly}
        tolyString := TalyToString(toly);
        if Axis = 'xScale' then
          begin
            Writeln(F, HR : 5 : 2, '  ', VT : 5 : 2, '  (', tolyString, ') showxhatch');
            HR := HR + STEP;
          end;

        if Axis = 'yScale' then
          begin
            Writeln(F, HR : 5 : 2, '  ', VT : 5 : 2, '  (', tolyString, ') showyhatch');
            VT := VT + STEP;
          end;

        toly := toly + taly;
      end; {while TOLY}

  end; {PostMarks: Marks and Numbers}

{_________________________________________________}

  procedure PostFrame (var F: text;
                  var dataBOX: rBOX;
                  var POSTBOX: IBOX);

    var
      sign, Ycap: integer;
      firstHATCH, xZERO, yZERO, vSTEP, hSTEP: real;
      rxPos, rxNeg, ryPos, ryNeg, taly, increment: real;
      nearBound, farBound, boundRange, startMark: real;

  begin{PostFrame}
    with POSTBOX, dataBOX do
      begin

        yRange := yMax - yMin;
        xRange := xMax - xMin;

{(X1,  Y1) is the bottom left of the IBOX}
{(X2,  Y2) is the top lright of the IBOX}

        xZERO := X1; {Default for Graphics Origin. integer to real conversion}
        yZERO := Y1; {Default for Graphics Origin. integer to real conversion}

        if doMarks then
          begin
            Writeln(F, '/Symbol findfont');
            Writeln(F, '10 scalefont  setfont');
          end;
        if doFrame then
          begin
            Writeln(F, X1 : 3, '  ', Y1 : 3, '  moveto');
            Writeln(F, X1 : 3, '  ', Y2 : 3, '  lineto');
            Writeln(F, X2 : 3, '  ', Y2 : 3, '  lineto');
            Writeln(F, X2 : 3, '  ', Y1 : 3, '  lineto');
            Writeln(F, 'closepath');
            Writeln(F, 'stroke');
            Writeln(F, 'newpath');
          end;

        if yScale = true then
          begin {Do the yScale}

 {...............................................................................................}

            if yMin * yMax > 0 then
              begin {yMin, yMax > or yMin, yMax < 0}

                if yMax < 0 then
                  begin
                    sign := -1;
                    nearBound := yMax;
                    farBound := yMin;
                  end
                else
                  begin
                    sign := 1;
                    nearBound := yMin;
                    farBound := yMax
                  end;

                if ((farBound - nearBound) / farBound) < 0.3 then
                  begin {small boundRange}

                    if hBorders then
                      begin
                        nearBound := nearBound - sign * (0.2 * yRange);
                        farBound := farBound + sign * (0.2 * yRange);
                        if sign = 1 then
                          begin
                            yUB := farBound;
                            YLB := nearBound;
                          end
                        else
                          begin
                            yLB := farBound;
                            YUB := nearBound;
                          end;
                      end; {hborders}

                    boundRange := Abs(farBound - nearBound);

                    TallyPoints(boundRange, taly);
                    startMark := Trunc(nearBound / taly) * taly;
                    if (Abs((startMark - nearBound) / nearBound) > 0.0001) then
                      startMark := startMark + sign * taly;

                    if sign = 1 then
                      Ycap := Y1
                    else
                      Ycap := Y2;

                    FIRSTHATCH := Ycap + (startMark - nearBound) * BOXHGT / boundRange;
                  end {small boundRange}

                else{large boundRange}
                  begin {yMin = 0.0}

                    if hBorders then {nearBound := 0.0;}
                      begin
                        if yMax > 0 then
                          begin
                            farBound := 1.2 * yMax;
                            yUB := farBound;
                            yLB := 0.0;
                          end
                        else
                          begin
                            farBound := 1.2 * yMin;
                            yLB := farBound;
                            yUB := 0.0;
                          end;
                        end; {hborders}

                    boundRange := Abs(farBound);
                    TallyPoints(boundRange, taly);
                    startMark := 0.0;
                    FIRSTHATCH := Ycap;
                  end; {large boundRange}

                vSTEP := taly * (BOXHGT / boundRange);
                hSTEP := 0.0;
                if doMarks then
                  PostMarks(F, X1, firstHATCH, sign * taly, startMark, farBound, sign * vSTEP, 'yScale')

              end;{yMin, yMax > or yMin, yMax < 0}
{...............................................................................................}

            if (yMin < 0.0) and (yMax > 0.0) then
              begin {yMin < 0.0}
                ryPos := yMax;
                ryNeg := yMin;

                if hBorders then
                  begin {hBorders}
                    increment := 0.1 * (yMax - yMin);
                    if yMax > Abs(yMin) then
                      begin
                        ryPos := 1.2 * yMax;
                        ryNeg := yMin - increment;
                      end
                    else
                      begin
                        ryNeg := 1.2 * yMin;
                        ryPos := yMax + increment
                      end;
                    yUB :=ryPos;
                    yLB := rYNeg;
                   end; {hBorders}

                yZERO := Y1 - BOXHGT * ryNeg / (ryPos - ryNeg);
{this draws the horizontal axis through y = 0}

                if (xScale = true) then
                  begin
                    Writeln(F, X1 : 3, '  ', yZERO : 5 : 2, '  moveto');
                    Writeln(F, X2 : 3, '  ', yZERO : 5 : 2, '  lineto')
                  end;

                if ryPos > Abs(ryNeg) then
                  begin
                    TallyPoints(ryPos, taly);
                    vSTEP := (Y2 - yZERO) * (taly / ryPos);
                  end
                else {if ryPos < Abs(ryNeg)}
                  begin
                    TallyPoints(-ryNeg, taly);
                    vSTEP := (Y1 - yZERO) * taly / ryNeg;
                  end;

                hSTEP := 0.0;
                if doMarks then
                  begin {doMarks}
                    PostMarks(F, X1, yZERO, taly, 0.0, ryPos, vSTEP, 'yScale');
                    PostMarks(F, X1, yZERO, -taly, 0.0, ryNeg, -vSTEP, 'yScale');
                  end; {doMarks}

              end; {yMin < 0.0 and yMax > 0.0}
  {...............................................................................................}

          end; {Do the yScale}
{------------------------------------------------------}

        if xScale then
          begin {Do the xScale}

  {...............................................................................................}

            if (xMin <= 0.0) and (xMax >= 0) then
              begin {xMin <= 0.0}
                rxPos := xMax;
                rxNeg := xMin;

                if (xMin < 0.0) then
                  xZERO := X1 - BOXWDT * rxNeg / (rxPos - rxNeg)
                else
                  xZERO := X1;

                if (xMin < 0.0) and (yScale = true) then
                  begin  {This provides the vertical axis through x =0}
                    Writeln(F, xZERO : 5 : 2, '  ', Y1 : 3, '  moveto');
                    Writeln(F, xZERO : 5 : 2, '  ', Y2 : 3, '  lineto');
                  end;

                if rxPos > Abs(rxNeg) then
                  begin
                    TallyPoints(rxPos, taly);
                    hSTEP := (X2 - xZERO) * (taly / rxPos);
                  end
                else
                  begin
                    TallyPoints(-rxNeg, taly);
                    hSTEP := (X1 - xZERO) * (taly / rxNeg);
                  end;

                vSTEP := 0;
                if doMarks then
                  begin
                    PostMarks(F, xZERO, Y1, taly, 0, rxPos, hSTEP, 'xScale');
                    PostMarks(F, xZERO, Y1, -taly, 0, rxNeg, -hSTEP, 'xScale');
                  end;

              end; {xMin <= 0.0 and xMax >= 0}

  {...............................................................................................}

            if xMin * xMax > 0.0 then
              begin {xMin, xMax < 0 or xMin, xMax >0}
                sign := 1;
                if xMax < 0 then
                  sign := -1;

                TallyPoints(xRange, taly);
                if xMin > 0 then
                  begin
                    farBound := xMax;
                    nearBound := xMin;
                  end;
                if xMax < 0 then
                  begin
                    farBound := xMin;
                    nearBound := xMax;
                  end;

                startMark := Trunc(nearBound / taly) * taly;
                if (Abs((startMark - nearBound) / nearBound) > 0.001) then
                  startMark := startMark + sign * taly;
                hSTEP := (BOXWDT) * (taly / xRange);
                vSTEP := 0.0;
                if sign = 1 then
                  firstHATCH := X1 + ((startMark - xMin) * BOXWDT) / xRange
                else
                  firstHATCH := X2 - ((xMax - startMark) * BOXWDT) / xRange;
                PostMarks(F, firstHATCH, Y1, sign * taly, startMark, farBound, sign * hSTEP, 'xScale');
              end;

  {...............................................................................................}

            Writeln(F, 'stroke');
            Writeln(F, 'newpath');

          end; {Do the xScale}

{------------------------------------------------------}
        X0 := Round(xZERO);
        Y0 := Round(yZERO);

      end; {with POSTBOX, dataBOX}

  end; {PostFrame: PostScript Framework}

{________________________________________________}

  procedure PostVertAxis (var F: text;
                  XSTART, YSTART: integer;
                  taly, firstMark, farBound, VSTEP: real);

    var
      toly, HR, VT: real;
      tolyString: string;

  begin {PostVertAxis}

    toly := firstMark;
    HR := XSTART;
    VT := YSTART;

    while Abs(toly) <= Abs(farBound) do
      begin
        tolyString := TalyToString(toly);
        Writeln(F, HR : 5 : 2, '  ', VT : 5 : 2, '  (', tolyString, ') showyhatch');
        VT := VT + VSTEP;
        toly := toly + taly;
      end;

  end; {PostVertAxis}

 {_________________________________________________}

  procedure PostZeroPiAxis (var F: text;
                  var POSTBOX: IBOX);

    var
      XR, YT, BOXWDT: real;

  begin{ZeroPiAxis}
    BOXWDT := POSTBOX.X2 - POSTBOX.X1;

    XR := POSTBOX.X1;
    YT := POSTBOX.Y1; {type conversion}
    Writeln(F, XR : 5 : 2, '  ', YT : 5 : 2, '  (0) showxhatch');

    XR := POSTBOX.X1 + BOXWDT / 4;
    Writeln(F, XR : 5 : 2, '  ', YT : 5 : 2, '  (p/4) showxhatch');

    XR := POSTBOX.X1 + BOXWDT / 2;
    Writeln(F, XR : 5 : 2, '  ', YT : 5 : 2, '  (p/2) showxhatch');

    XR := POSTBOX.X2 - BOXWDT / 4;
    Writeln(F, XR : 5 : 2, '  ', YT : 5 : 2, '  (3p/4) showxhatch');

    XR := POSTBOX.X2;
    Writeln(F, XR : 5 : 2, '  ', YT : 5 : 2, '  (p) showxhatch');
    Writeln(F, 'stroke');
    Writeln(F, 'newpath');

  end;{PostZeroPiAxis}

{_________________________________________________}

  procedure PostPolyPiAxis (var F: text;
                  var POSTBOX: IBOX;
                  var spectraBOX: rBOX);

    var
      XR, YT, BOXWDT: real;
      piMultiple: integer;
      pi, increment, addSum, piSum, piSize: real;
      hatchPi, hatchText, hatchString: string;

  begin
    pi := 4.0 * ArcTan(1.0);
    piMultiple := Round(spectraBOX.xMax / pi);
       {piSize and piSum are fractions or multiples of pi}

    BOXWDT := POSTBOX.X2 - POSTBOX.X1;
    XR := POSTBOX.X1;
    YT := POSTBOX.Y1; {type conversion}

    addSum := 0.0;
    piSum := 0.0;

    if piMultiple in [1, 2, 4, 8] then
      begin
        increment := BOXWDT / 4.0;
        piSize := piMultiple / 4.0;
      end;

    if piMultiple in [3, 6, 7, 9] then
      begin
        increment := BOXWDT / 3.0;
        piSize := piMultiple / 3.0;
      end;

    if piMultiple in [5, 10] then
      begin
        increment := BOXWDT / 5.0;
        piSize := piMultiple / 5.0;
      end;

    if piMultiple = 7 then
      begin
        increment := 2.0 * BOXWDT / 7.0;
        piSize := 2.0 * piMultiple / 7.0;
      end;

    while XR <= POSTBOX.X2 do
      begin {while}

        if (piSum = 0.0) then
          hatchText := '(0)'
        else if (Round(piSum * 100) = 25) then
          hatchText := '(p/4)'
        else if (Round(piSum * 100) = 50) then
          hatchText := '(p/2)'
        else if (Round(piSum * 100) = 75) then
          hatchText := '(3p/4)'
        else if (Round(piSum * 100) = 100) then
          hatchText := '(p)'
        else if (Round(piSum * 100) = 150) then
          hatchText := '(3p/2)'
        else
          begin
            hatchPi := TalyToString(Round(piSum));
            hatchText := Concat('(', hatchPi, 'p', ')')
          end;

        hatchString := Concat(' ', hatchText, ' showxhatch');
        Writeln(F, XR : 5 : 2, '  ', YT : 5 : 2, hatchString);

        AddSum := AddSum + increment;
        piSum := piSum + piSize;
        XR := POSTBOX.X1 + addSum;
      end;   {while}

  end; {PostPolyPiAxis}

{________________________________________________}

  procedure PostFrameLite (var F: text;
                  var realBOX: rBOX;
                  var POSTBOX: IBOX);

    var
      BOXHGT: integer;
      VSTEP, taly: real;
      lowerBound, upperBound: real;

  begin{PostFrameLite}
    BOXHGT := POSTBOX.Y2 - POSTBOX.Y1;

    Writeln(F, '/Symbol findfont');
    Writeln(F, '10 scalefont  setfont');
    Writeln(F, POSTBOX.X1 : 2, '  ', POSTBOX.Y1 : 2, '  moveto');
    Writeln(F, POSTBOX.X1 : 2, '  ', POSTBOX.Y2 : 3, '  lineto');
    Writeln(F, POSTBOX.X2 : 3, '  ', POSTBOX.Y2 : 3, '  lineto');
    Writeln(F, POSTBOX.X2 : 2, '  ', POSTBOX.Y1 : 3, '  lineto');
    Writeln(F, 'closepath');
    Writeln(F, 'stroke');
    Writeln(F, 'newpath');

    lowerBound := 0.0;
    upperBound := 1.2 * realBOX.yMax;
    TallyPoints(upperBound, taly);

    VSTEP := taly * (BOXHGT / upperBound);

    PostVertAxis(F, POSTBOX.X1, POSTBOX.Y1, taly, lowerBound, upperBound, VSTEP);
    Writeln(F, 'stroke');
    Writeln(F, 'newpath');

    realBOX.yUB := upperBound;
    realBOX.yLB := lowerBound;
    realBOX.xLB := realBOX.xMin;
    realBOX.xRB := realBOX.xMax;

    POSTBOX.X0 := POSTBOX.X1;
    POSTBOX.Y0 := POSTBOX.Y1;

  end; {PostFrameLite}

{________________________________________________}

  procedure AutoPostFrame (var F: text;
                  lags: integer;
                  var POSTBOX: IBOX;
                  autoMin: real;
                  var autoRange: real);

    var
      BOXWDT, BOXHGT: integer;
      XZERO, YZERO, HSTEP, VT: real;
      xRange, subRange, taly, toly: real;
      tolyString: string;

  begin{AutoPostFrame}
    BOXHGT := POSTBOX.Y2 - POSTBOX.Y1;
    BOXWDT := POSTBOX.X2 - POSTBOX.X1;

    taly := 0.25;

    if autoMin = 0.0 then
      begin
        subRange := 0;
        autoRange := 1.15; {Provide some 15% headroom}
      end
    else
      begin {Determine the negative range of y}
        subRange := (1 + Trunc(autoMin / 0.25)) * 0.25;
        autoRange := subRange + 1.30;
      end; {autoRange = total range plus headroom and footroom}

    if autoMin = 0.0 then
      YZERO := POSTBOX.Y1
    else
      YZERO := Round(((subRange + 0.15) * BOXHGT) / autoRange);
    XZERO := POSTBOX.X1;

    BlackDotDef(F);

    Writeln(F, '/Symbol findfont');
    Writeln(F, '10 scalefont  setfont');

    Writeln(F, POSTBOX.X1 : 3, '  ', POSTBOX.Y1 : 3, '  moveto');
    Writeln(F, POSTBOX.X1 : 3, '  ', POSTBOX.Y2 : 3, '  lineto');
    Writeln(F, POSTBOX.X2 : 3, '  ', POSTBOX.Y2 : 3, '  lineto');
    Writeln(F, POSTBOX.X2 : 3, '  ', POSTBOX.Y1 : 3, '  lineto');
    Writeln(F, 'closepath');
    Writeln(F, XZERO : 5 : 2, '  ', YZERO : 5 : 2, '  moveto');
    Writeln(F, BOXWDT : 3, '  ', YZERO : 5 : 2, '  lineto');

    VT := YZERO;
    toly := 0.0;

    while toly <= 1.2 do
      begin
        tolyString := TalyToString(toly);
        Writeln(F, XZERO : 5 : 2, '  ', VT : 5 : 2, '  (', tolyString, ') showyhatch');
        toly := toly + taly;
        VT := YZERO + BOXHGT * (toly / autoRange);
      end;

    toly := taly;

    if autoMin <> 0 then
      while toly <= subRange + 0.2 do
        begin
          VT := YZERO - BOXHGT * (toly / autoRange);
          tolyString := TalyToString(-toly);
          Writeln(F, XZERO : 5 : 2, '  ', VT : 5 : 2, '  (', tolyString, ') showyhatch');
          toly := toly + taly;
        end;

    xRange := lags + 1;
    TallyPoints(xRange, taly);
    HSTEP := (BOXWDT * taly / xRange);
    PostMarks(F, XZERO, POSTBOX.X1, taly, 0, xRange, HSTEP, 'xScale');

    Writeln(F, 'stroke');

    POSTBOX.X0 := Round(XZERO);
    POSTBOX.Y0 := Round(YZERO);

  end; {AutoPostFrame}

{________________________________________________}

  procedure PostSpikes (var F: text;
                  lags: integer;
                  var POSTBOX: IBOX;
                  var rho: shortVector;
                  autoRange: real);

    var
      i, BOXWDT, BOXHGT: integer;
      HR, VT: real;

  begin {PostSpikes}
    BOXHGT := POSTBOX.Y2 - POSTBOX.Y1;
    BOXWDT := POSTBOX.X2 - POSTBOX.X1;

    Writeln(F, '2.0 setlinewidth');

    HR := POSTBOX.X0;
    VT := POSTBOX.Y0 + BOXHGT / autoRange;
    Writeln(F, HR : 3 : 2, '  ', VT : 3 : 2, ' 1.0  blackdot');

    for i := 1 to lags do
      begin
        HR := POSTBOX.X0 + BOXWDT * i / (lags + 1);
        VT := POSTBOX.Y0 + BOXHGT * (rho[i] / autoRange);
        Writeln(F, HR : 5 : 2, '  ', POSTBOX.Y0 : 3, '  moveto');
        Writeln(F, HR : 5 : 2, '  ', VT : 5 : 2, '  lineto');
      end;
    Writeln(F, 'stroke');

  end; {PostSpikes}

{_________________________________________________}

  procedure PostPlot (var F: text;
                  var realBOX: rBOX;
                  var POSTBOX: IBOX;
                  var dataVector: longVector);
    var
      i: integer;
      XZERO, YZERO, BOXWDT, BOXHGT: real;
      HR, VT, xRange, yRange: real;

  begin
    BOXHGT := POSTBOX.Y2 - POSTBOX.Y1;
    BOXWDT := POSTBOX.X2 - POSTBOX.X1;
    XZERO := POSTBOX.X0;
    YZERO := POSTBOX.Y0;
    yRange := realBOX.yUB - realBOX.yLB;
   {xRange := realBOX.xRB - realBOX.xLB;}
    xRange := realBOX.NOPUNCT - 1;

    for i := 0 to realBOX.NOPUNCT - 1 do
      begin {i}
        HR := XZERO + (i * BOXWDT / xRange);
        VT := YZERO + (dataVector[i] * BOXHGT / yRange);
        if i = 0 then
          Writeln(F, HR : 3 : 2, '  ', VT : 3 : 2, '   moveto')
        else
          Writeln(F, HR : 3 : 2, '  ', VT : 3 : 2, '   lineto');
      end; {i}
    Writeln(F, 'stroke');
    Writeln(F, 'newpath');

  end; {PostPlot}

 {________________________________________________}

  procedure PostPlot2 (var F: text;
                  var realBOX: rBOX;
                  var POSTBOX: IBOX;
                  var dataVector: longVector);

 {This is for plotting an arbitrary number of data points over}
 {the range defined by the width of the bounding box via the}
 {index  i := 0 to Tcap - 1: Tcap := realBOX.NOPUNCT - 1}

    var
      i, Tcap: integer;
      XZERO, YZERO, BOXWDT, BOXHGT: real;
      HR, VT, xRange, yRange: real;

  begin
    BOXHGT := POSTBOX.Y2 - POSTBOX.Y1;
    BOXWDT := POSTBOX.X2 - POSTBOX.X1;
    XZERO := POSTBOX.X0;
    YZERO := POSTBOX.Y0;
    yRange := realBOX.yUB - realBOX.yLB;
    xRange := realBOX.NOPUNCT - 1; {real value}
    Tcap := realBOX.NOPUNCT - 1; {Integer Value}
    Writeln(F, 'newpath');
    HR := BOXWDT;
    VT := YZERO + (dataVector[Tcap - 1] * BOXHGT / yRange);
    Writeln(F, HR : 3 : 2, ' ', VT : 3 : 2, ' moveto');

    for i := 0 to Tcap - 1 do
      begin {i}
        HR := XZERO + (i * BOXWDT / xRange);
        VT := YZERO + (dataVector[i] * BOXHGT / yRange);
        if (i mod 8 = 7) or (i = Tcap - 1) then
          Writeln(F, HR : 3 : 2, ' ', VT : 3 : 2)
        else
          Write(F, HR : 3 : 2, ' ', VT : 3 : 2, ' ');
      end; {i}
    Writeln(F, Tcap : 3, ' {lineto} repeat');
    Writeln(F, 'stroke');

  end; {PostPlot2}

 {_________________________________________________}

  procedure PostBiPlot (var F: text;
                  var realBOX: rBOX;
                  var POSTBOX: IBOX;
                  var dataVector: pointsVector);
    var
      i, BOXWDT, BOXHGT: integer;
      XZERO, YZERO: real;
      HR, VT, xRange, yRange: real;

  begin
    BOXHGT := POSTBOX.Y2 - POSTBOX.Y1;
    BOXWDT := POSTBOX.X2 - POSTBOX.X1;
    XZERO := POSTBOX.X0;
    YZERO := POSTBOX.Y0;
    yRange := realBOX.yUB - realBOX.yLB;
    xRange := realBOX.xRB - realBOX.xLB;

    for i := 0 to realBOX.NOPUNCT - 1 do
      begin {i}
         HR := XZERO + (dataVector[i].x * BOXWDT / xRange);
        VT := YZERO + (dataVector[i].y * BOXHGT / yRange);
        if i = 0 then
          Writeln(F, HR : 3 : 2, '  ', VT : 3 : 2, '   moveto')
        else
          Writeln(F, HR : 3 : 2, '  ', VT : 3 : 2, '   lineto');
      end; {i}
    Writeln(F, 'stroke');
    Writeln(F, 'newpath');

  end; {PostBiPlot}

 {_________________________________________________}

  procedure PostPlotMod (var F: text;
                  var realBOX: rBOX;
                  var POSTBOX: IBOX;
                  var dataVector: longVector);

 {This is for plotting a series of function evaluations that are equal}
 {in number to the width to the bounding box measured in pixelpoints}
 {via the index  i := 0 to BOXWDT - 1}

    var
      i, BOXWDT, BOXHGT, YZERO: integer;
      HR, VT, yRange: real;

  begin
    BOXWDT := POSTBOX.X2 - POSTBOX.X1;
    BOXHGT := POSTBOX.Y2 - POSTBOX.Y1;
    YZERO := POSTBOX.Y0;
    yRange := realBOX.yUB - realBOX.yLB;

    Writeln(F, 'newpath');
    Writeln(F, '0.5 setlinewidth');
    Writeln(F, '/abscissa ', BOXWDT : 3, ' def');

    HR := BOXWDT;
    VT := YZERO + (dataVector[BOXWDT - 1] * BOXHGT / yRange);
    Writeln(F, HR : 3 : 2, ' ', VT : 3 : 2, ' moveto');
 {we are plotting in the reverse direction of x}

    for i := 0 to BOXWDT - 1 do
      begin {i}
        VT := YZERO + (dataVector[i] * BOXHGT / yRange);
 {Here we  impose a limiting value on VT to keep the plot within the  bounding box}
        if VT > POSTBOX.Y2 then
          VT := POSTBOX.Y2;

        if (i mod 8 = 7) then
          Writeln(F, VT : 3 : 2) {this is to break the lines}
        else
          Write(F, VT : 3 : 2, ' ');
      end; {i}
    Write(F, BOXWDT : 3); {repeat these many times}
    Writeln(F, ' {/abscissa abscissa 1 sub def abscissa exch lineto} repeat');
    Writeln(F, 'stroke');

  end; {PostPlotMod}

{_________________________________________________}

  procedure WritePostFile (var dataBOX: rBOX;
                  var POSTBOX: IBOX;
                  var dataVector: longVector;
                  var legend: string);
    var
      F: text;
      postFileName: string;

  begin

    Write('    Name the PostScript File = ');
    Readln(postFileName);
    if postFileName <> '' then
      begin {Postfile}
        Assign(F, PostfileName);
        Rewrite(F);
        PostHeader(F, POSTBOX, TeXFormat);
        PostFrame(F, dataBOX, POSTBOX);
        PostPlot(F, dataBOX, POSTBOX, dataVector);
        PostFooter(F, POSTBOX, legend, TeXFormat);
        Close(F);
      end; {Postfile}
    figNumber := figNumber + 1;
  end; {WritePostFile}

  {_________________________________________________}

end. {POST: Unit 6 of CONCRETE.PAS}

