unit SCREEN; {Unit 5 of CONCRETE.PAS}

interface

  uses
    GLOBALS, UTILS, MATHS, LEGENDS, Graph, Dos, Crt;

  procedure MarksAndNumbers (XSTART, YSTART: integer;
                  taly, firstMark, farBound, STEP: real;
                  Axis: string);

  procedure ScreenFrame (var dataBOX: rBOX;
                  var SCREENBOX: IBOX);

  procedure ScreenFrameLite (var dataBOX: rBOX;
                  var SCREENBOX: IBOX);

  procedure VertAxis (XSTART, YSTART: integer;
                  taly, firstMark, farBound, VSTEP: real);

  procedure ZeroPiAxis (var SCREENBOX: IBOX);

  procedure PolyPiAxis (var SCREENBOX: IBOX;
                  var spectraBOX: rBOX);

  procedure ScreenPlot (var realBOX: rBOX;
                  var SCREENBOX: IBOX;
                  var dataVector: longVector);

  procedure ScreenPlotMod (var realBOX: rBOX;
                  var SCREENBOX: IBOX;
                  var dataVector: longVector);

  procedure ScreenBiPlot (var realBOX: rBOX;
                  var SCREENBOX: IBOX;
                  var dataVector: pointsVector);

  procedure AutoFrame (lags: integer;
                  var autoBOX: rBOX;
                  var SCREENBOX: IBOX;
                  autoMin: real;
                  var autoRange: real);

  procedure AutoSpikes (lags: integer;
                  var autoBOX: rBOX;
                  var SCREENBOX: IBOX;
                  rho: shortVector;
                  autoRange: real);

implementation

{_________________________________________________}

  procedure MarksAndNumbers (XSTART, YSTART: integer;
                  taly, firstMark, farBound, STEP: real;
                  Axis: string);

    var
      toly, HRL, VRL: real;
      HR1, HR2, VT1, VT2, tolyWidth: integer;
      HINC, VINC, PENLEFT, PENDOWN: integer;
      tolyString: string;

  begin {MarksAndNumbers}

    if Axis = 'yScale' then
      begin {Pixel Parameters}
        HINC := 5;
        VINC := 0;
        PENDOWN := 4;
      end;
    if Axis = 'xScale' then
      begin
        HINC := 0;
        VINC := -5;
        PENDOWN := 20;
      end; {Pixel Parameters}

    toly := firstMark;
    HRL := XSTART;
    HR1 := XSTART;
    HR2 := HR1 + HINC;
    VRL := YSTART;
    VT1 := YSTART;
    VT2 := VT1 + VINC;

    while Abs(toly) <= Abs(farBound) do

      begin {while toly}
        tolyString := TalyToString(toly);
        tolyWidth := TextWidth(tolyString);
        if axis = 'xScale' then
          PENLEFT := Round(tolyWidth / 2);
        if axis = 'yScale' then
          PENLEFT := tolyWidth + 15;

 {Numbers}
        MoveTo(HR1 - PENLEFT, VT1 + PENDOWN);
        OutText(tolyString);

  {Hatchmarks}
        MoveTo(HR1, VT1);
        LineTo(HR2, VT2);

  {Increments}
        if axis = 'xScale' then
          HRL := HRL + STEP;
        HR1 := Round(HRL);
        HR2 := HR1 + HINC;
        if axis = 'yScale' then
          VRL := VRL - STEP;
        VT1 := Round(VRL);
        VT2 := VT1 + VINC;

        toly := toly + taly;
      end; {while toly}

  end; {MarksAndNumbers}

{_________________________________________________}

  procedure ScreenFrame (var dataBOX: rBOX;
                  var SCREENBOX: IBOX);


{Uses, MarksAndNumbers, TallyPoints, TalyToString}

    var
      FIRSTHATCH: integer;
      XORIGIN, YORIGIN, YCap, sign: integer;
      rxPos, rxNeg, ryPos, ryNeg, vSTEP, hSTEP, taly: real;
      nearBound, farBound, boundRange, increment, startMark: real;

  begin

    with SCREENBOX, dataBOX do
      begin

{(X1,  Y1) is the top left of the IBOX}
{(X2,  Y2) is the botton lright of the IBOX}

        XORIGIN := X1;{ Default for  XORIGIN}
        YORIGIN := Y2;{ Default for  YORIGIN}

        if doFrame then
          Rectangle(X1, Y1, X2 + 1, Y2 + 1);

        if yScale = true then
          begin {Do the yScale}

{...............................................................................................}

            if yMin * yMax > 0 then
              begin {yMin, yMax > or yMin, yMax < 0}

                if yMax < 0 then
                  begin
                    sign := -1;
                    nearBound := yMax;
                    farBound := yMin;
                  end
                else
                  begin
                    sign := 1;
                    nearBound := yMin;
                    farBound := yMax
                  end;

                if ((farBound - nearBound) / farBound) < 0.3 then
                  begin {small boundRange}

                    if hBorders then
                      begin
                        nearBound := nearBound - sign * (0.2 * yRange);
                        farBound := farBound + sign * (0.2 * yRange);
                        if sign = 1 then
                          begin
                            yUB := farBound;
                            YLB := nearBound;
                          end
                        else
                          begin
                            yLB := farBound;
                            YUB := nearBound;
                          end;
                      end; {hborders}

                    boundRange := Abs(farBound - nearBound);

                    TallyPoints(boundRange, taly);
                    startMark := Trunc(nearBound / taly) * taly;
                    if (Abs((startMark - nearBound) / nearBound) > 0.0001) then
                      startMark := startMark + sign * taly;

                    if sign = 1 then
                      Ycap := Y2
                    else
                      Ycap := Y1;

                    FIRSTHATCH := Ycap + Round((nearBound - startMark) * BOXHGT / boundRange);
                  end {small boundRange}

                else{large boundRange}
                  begin {yMin = 0.0}

                    if hBorders then {nearBound := 0.0;}
                      begin
                        if yMax > 0 then
                          begin
                            farBound := 1.2 * yMax;
                            yUB := farBound;
                            yLB := 0.0;
                          end
                        else
                          begin
                            farBound := 1.2 * yMin;
                            yLB := farBound;
                            yUB := 0.0;
                          end;
                        end; {hborders}

                    boundRange := Abs(farBound);
                    TallyPoints(boundRange, taly);
                    startMark := 0.0;
                    FIRSTHATCH := Ycap;
                  end; {large boundRange}

                VSTEP := taly * (BOXHGT / boundRange);
                HSTEP := 0.0;
                if doMarks then
                  MarksAndNumbers(X1, FIRSTHATCH, sign * taly, startMark, farBound, sign * VSTEP, 'yScale')

              end;{xMin, xMax > or xMin, xMax < 0}
{...............................................................................................}

            if (yMin < 0.0) and (yMax > 0.0) then
              begin {yMin < 0.0}
                ryPos := yMax;
                ryNeg := yMin;

                if hBorders then
                  begin {hBorders}
                    increment := 0.1 * (yMax - yMin);
                    if yMax > Abs(yMin) then
                      begin
                        ryPos := 1.2 * yMax;
                        ryNeg := yMin - increment;
                      end
                    else
                      begin
                        ryNeg := 1.2 * yMin;
                        ryPos := yMax + increment;
                      end;
                    yUB :=ryPos;
                    yLB := rYNeg;
                  end; {hBorders}

                YORIGIN := Y2 + Round(BOXHGT * ryNeg / (ryPos - ryNeg));
{this draws the horizontal axis through y = 0}

                if (xAxis = true) then
                  begin
                    MoveTo(X1, YORIGIN);
                    LineTo(X2, YORIGIN)
                  end;

                if ryPos > Abs(ryNeg) then
                  begin
                    TallyPoints(ryPos, taly);
                    VSTEP := (YORIGIN - Y1) * (taly / ryPos);
                  end
                else {if ryPos < Abs(ryNeg)}
                  begin
                    TallyPoints(-ryNeg, taly);
                    VSTEP := (YORIGIN - Y2) * (taly / ryNeg);
                  end;

                HSTEP := 0.0;
                if doMarks then
                  begin {doMarks}
                    MarksAndNumbers(X1, YORIGIN, taly, 0.0, ryPos, VSTEP, 'yScale');
                    MarksAndNumbers(X1, YORIGIN, -taly, 0.0, ryNeg, -VSTEP, 'yScale');
                  end; {doMarks}

              end; {yMin < 0.0 and yMax > 0.0}
{...............................................................................................}

          end; {Do the yScale}

{-----------------------------------------------}

        if xScale then
          begin {Do the xScale}

{...............................................................................................}

            if (xMin <= 0.0) and (xMax >= 0) then
              begin {xMin <= 0.0}
                rxPos := xMax;
                rxNeg := xMin;

                if (xMin < 0.0) then
                  XORIGIN := X1 - Round(BOXWDT * rxNeg / (rxPos - rxNeg))
                else
                  XORIGIN := X1;

                if (xMin < 0.0) and (yAxis = true) then
                  begin  {This provides the vertical axis through x =0}
                    MoveTo(XORIGIN, Y1);
                    LineTo(XORIGIN, Y2)
                  end;

                if rxPos > Abs(rxNeg) then
                  begin
                    TallyPoints(rxPos, taly);
                    HSTEP := (X2 - XORIGIN) * (taly / rxPos);
                  end
                else
                  begin
                    TallyPoints(-rxNeg, taly);
                    HSTEP := (X1 - XORIGIN) * (taly / rxNeg);
                  end;

                VSTEP := 0;
                if xMax > 0.0 then
                  MarksAndNumbers(XORIGIN, Y2, taly, 0, rxPos, HSTEP, 'xScale');
                if xMin < 0.0 then
                  MarksAndNumbers(XORIGIN, Y2, -taly, 0, rxNeg, -HSTEP, 'xScale');
              end; {xMin <= 0.0}

{...............................................................................................}

            if xMin * xMax > 0.0 then
              begin {xMin, xMax < 0 or xMin, xMax >0}
                sign := 1;
                if xMax < 0 then
                  sign := -1;

                TallyPoints(xRange, taly);
                if xMin > 0 then
                  begin
                    farBound := xMax;
                    nearBound := xMin;
                  end;
                if xMax < 0 then
                  begin
                    farBound := xMin;
                    nearBound := xMax;
                  end;

                startMark := Trunc(nearBound / taly) * taly;
                if (Abs((startMark - nearBound) / nearBound) > 0.0001) then
                  startMark := startMark + sign * taly;
                HSTEP := (BOXWDT) * (taly / xRange);
                VSTEP := 0.0;
                if sign = 1 then
                  FIRSTHATCH := X1 + Round((startMark - xMin) * BOXWDT / xRange)
                else
                  FIRSTHATCH := X2 - Round((xMax - startMark) * BOXWDT / xRange);
                MarksAndNumbers(FIRSTHATCH, Y2, sign * taly, startMark, farBound, sign * HSTEP, 'xScale');
              end;
{...............................................................................................}

          end; {Do the xScale}

{-----------------------------------------------}
        X0 := XORIGIN;
        Y0 := YORIGIN;

      end; {with SCREENBOX and dataBOX}

  end; {ScreenFrame}

{________________________________________________}

  procedure ScreenFrameLite (var dataBOX: rBOX;
                  var SCREENBOX: IBOX);

    var
      BOXHGT: integer;
      VSTEP, taly: real;
      lowerBound, upperBound: real;

  begin
    BOXHGT := SCREENBOX.Y2 - SCREENBOX.Y1; {Height of Bounding Box}

    Rectangle(SCREENBOX.X1, SCREENBOX.Y1, SCREENBOX.X2, SCREENBOX.Y2);

    lowerBound := 0.0;
    upperBound := 1.2 * dataBOX.yMax;
    TallyPoints(upperBound, taly);

    VSTEP := taly * (BOXHGT / upperBound);
    VertAxis(SCREENBOX.X1, SCREENBOX.Y2, taly, lowerBound, upperBound, VSTEP);

    dataBOX.yUB := upperBound;
    dataBOX.yLB := lowerBound;
    dataBOX.xLB := dataBOX.xMin;
    dataBOX.xRB := dataBOX.xMax;

    SCREENBOX.X0 := SCREENBOX.X1;
    SCREENBOX.Y0 := SCREENBOX.Y2;

  end; {ScreenFrameLite}

{_________________________________________________}

  procedure VertAxis (XSTART, YSTART: integer;
                  taly, firstMark, farBound, VSTEP: real);

    var
      toly, VRL: real;
      HR1, HR2, VT, tolyWidth: integer;
      HINC, PENLEFT, PENDOWN: integer;
      tolyString: string;

  begin {MarksandNumbers}

    HINC := 5;
    PENDOWN := 4;

    toly := firstMark;
    HR1 := XSTART;
    HR2 := HR1 + HINC;
    VRL := YSTART;
    VT := YSTART;

    while Abs(toly) <= Abs(farBound) do
      begin {while toly}

        tolyString := TalyToString(toly);
        tolyWidth := TextWidth(tolyString);
        PENLEFT := tolyWidth + 15;

 {Numbers}
        MoveTo(HR1 - PENLEFT, VT + PENDOWN);
        OutText(tolyString);

  {Hatchmarks}
        MoveTo(HR1, VT);
        LineTo(HR2, VT);

  {Increments}
        VRL := VRL - VSTEP;
        VT := Round(VRL);

        toly := toly + taly;
      end; {while toly}

  end; {VertAxis}

{_________________________________________________}

  procedure ZeroPiAxis (var SCREENBOX: IBOX);

    var
      HR, VT1, VT2: integer;
      BOXWDT, PENDOWN, PENLEFT: integer;

  begin{ZeroPiAxis}
    BOXWDT := SCREENBOX.X2 - SCREENBOX.X1;
    PENLEFT := 12;
    PENDOWN := 20;

    VT1 := SCREENBOX.Y2;
    VT2 := VT1 - 5;
    HR := SCREENBOX.X1;
    MoveTo(HR - 2, VT1 + PENDOWN);
    OutText('0');

    HR := SCREENBOX.X1 + Round(BOXWDT / 4);
    MoveTo(HR, VT1);
    LineTo(HR, VT2);
    MoveTo(HR - 8, VT1 + PENDOWN);
    OutText('p/4');

    HR := SCREENBOX.X1 + Round(BOXWDT / 2);
    MoveTo(HR, VT1);
    LineTo(HR, VT2);
    MoveTo(HR - 8, VT1 + PENDOWN);
    OutText('p/2');

    HR := SCREENBOX.X2 - Round(BOXWDT / 4);
    MoveTo(HR, VT1);
    LineTo(HR, VT2);
    MoveTo(HR - PENLEFT, VT1 + PENDOWN);
    OutText('3p/4');

    HR := SCREENBOX.X2;
    MoveTo(HR - 2, VT1 + PENDOWN);
    OutText('p');

  end;{ZeroPiAxis}

{_________________________________________________}

  procedure PolyPiAxis (var SCREENBOX: IBOX;
                  var spectraBOX: rBOX);

    var
      HR, VT1, VT2: integer;
      BOXWDT, PENLEFT, PENDOWN: integer;
      piMultiple, hatchTextWidth: integer;
      pi, increment, addSum, piSum, piSize: real;
      hatchPi, hatchText: string;

  begin
    pi := 4.0 * ArcTan(1.0);
    piMultiple := Round(spectraBOX.xMax / pi);
       {piSize and piSum are fractions or multiples of pi}

    BOXWDT := SCREENBOX.X2 - SCREENBOX.X1;
    PENDOWN := 20;
    VT1 := SCREENBOX.Y2;
    VT2 := VT1 - 5;
    addSum := 0.0;
    piSum := 0.0;

    if piMultiple in [1, 2, 4, 8] then
      begin
        increment := BOXWDT / 4.0;
        piSize := piMultiple / 4.0;
      end;

    if piMultiple in [3, 6, 7, 9] then
      begin
        increment := BOXWDT / 3.0;
        piSize := piMultiple / 3.0;
      end;

    if piMultiple in [5, 10] then
      begin
        increment := BOXWDT / 5.0;
        piSize := piMultiple / 5.0;
      end;

    if piMultiple = 7 then
      begin
        increment := 2.0 * BOXWDT / 7.0;
        piSize := 2.0 * piMultiple / 7.0;
      end;

       {Do the hatchmarks}
    HR := SCREENBOX.X1;
    while HR <= SCREENBOX.X2 do
      begin {while}
        MoveTo(HR, VT1);
        LineTo(HR, VT2);

        if (piSum = 0.0) then
          hatchText := '0'
        else if (Round(piSum * 100) = 25) then
          hatchText := 'p/4'
        else if (Round(piSum * 100) = 50) then
          hatchText := 'p/2'
        else if (Round(piSum * 100) = 75) then
          hatchText := '3p/4'
        else if (Round(piSum * 100) = 100) then
          hatchText := 'p'
        else if (Round(piSum * 100) = 150) then
          hatchText := '3p/2'
        else
          begin
            hatchPi := TalyToString(Round(piSum));
            hatchText := Concat(hatchPi, 'p')
          end;

        hatchTextWidth := TextWidth(hatchText);
        PENLEFT := Round(hatchTextWidth / 2.0);
        MoveTo(HR - PENLEFT, VT1 + PENDOWN);
        OutText(hatchText);

        AddSum := AddSum + increment;
        piSum := piSum + piSize;
        HR := SCREENBOX.X1 + Round(AddSum);
      end;   {while}

  end; {PolyPiAxis}

   {________________________________________________}

  procedure ScreenPlot (var realBOX: rBOX;
                  var SCREENBOX: IBOX;
                  var dataVector: longVector);

    var
      i, HR, VT, BOXWDT, BOXHGT: integer;
      yRange: real;
      store: double;

   {The assumptiion is that all are indexed from zero to NOPUNCT-1}
  begin
    BOXWDT := SCREENBOX.X2 - SCREENBOX.X1;
    BOXHGT := SCREENBOX.Y2 - SCREENBOX.Y1;
    yRange := realBOX.yUB - realBOX.yLB;

    for i := 0 to realBOX.NOPUNCT - 1 do
      begin {i}
        store := i * BOXWDT;
        store := store / (realBOX.NOPUNCT - 1);
        HR := SCREENBOX.X1 + Round(Store);

        store := (dataVector[i] - realBOX.yLB) / yRange;
        VT := Round(SCREENBOX.Y2 - store * BOXHGT);

        if i = 0 then
          MoveTo(HR, VT)
        else
          LineTo(HR, VT);
      end; {i}

  end; {ScreenPlot}

   {_________________________________________________}

  procedure ScreenPlotMod (var realBOX: rBOX;
                  var SCREENBOX: IBOX;
                  var dataVector: longVector);
    var
      i, HR, VT, BOXWDT, BOXHGT: integer;
      XORIGIN, YORIGIN: integer;
      yRange: real;

  begin
    BOXWDT := SCREENBOX.X2 - SCREENBOX.X1;
    BOXHGT := SCREENBOX.Y2 - SCREENBOX.Y1;
    XORIGIN := Round(SCREENBOX.X0);
    YORIGIN := Round(SCREENBOX.Y0);
    yRange := realBOX.yUB - realBOX.yLB;

    for i := 0 to BOXWDT-1 do
      begin {i}
        HR := XORIGIN + i;
        VT := YORIGIN - Round(dataVector[i] * BOXHGT / yRange);
        if i = 0 then
          MoveTo(HR, VT)
        else
          LineTo(HR, VT);
      end; {i}
  end; {ScreenPlotMod}

  {________________________________________________}

  procedure ScreenBiPlot (var realBOX: rBOX;
                  var SCREENBOX: IBOX;
                  var dataVector: pointsVector);

    {This procedure  is used when dataVector is of the pointsVector}
    {type as opposed to tthe usual longVector type}

    var
      i, HR, VT, BOXWDT, BOXHGT: integer;
      xRange, yRange: real;
      store: double;

  begin
    BOXWDT := SCREENBOX.X2 - SCREENBOX.X1;
    BOXHGT := SCREENBOX.Y2 - SCREENBOX.Y1;
    yRange := realBOX.yUB - realBOX.yLB;
    xRange := realBOX.xRB - realBOX.xLB;

    for i := 0 to realBOX.NOPUNCT - 1 do
      begin {i}
        store := (dataVector[i].x - realBOX.xLB) / xRange;
        HR := Round(SCREENBOX.X1 + store * BOXWDT);

        store := (dataVector[i].y - realBOX.yLB) / yRange;
        VT := Round(SCREENBOX.Y2 - store * BOXHGT);

        if i = 0 then
          MoveTo(HR, VT)
        else
          LineTo(HR, VT);
      end; {i}

  end; {ScreenBiPlot}

 {________________________________________________}

  procedure AutoFrame (lags: integer;
                  var autoBOX: rBOX;
                  var SCREENBOX: IBOX;
                  autoMin: real;
                  var autoRange: real);

    var
      BOXWDT, BOXHGT, XORIGIN, YORIGIN: integer;
      xRange, subRange, HSTEP, taly, toly: real;
      VT, HR, PENLEFT, PENDOWN, tolyWidth: integer;
      tolyString: string;

  begin{AutoFrame}

    BOXWDT := SCREENBOX.X2 - SCREENBOX.X1;
    BOXHGT := SCREENBOX.Y2 - SCREENBOX.Y1;

    taly := 0.25;
    PENLEFT := 30;
    PENDOWN := -(TextHeight('NUMBERS') div 2);

    if autoMin = 0.0 then
      begin
        subRange := 0;
        autoRange := 1.15; {Provide some 15% headroom}
      end
    else
      begin {Determine the negative range of y}
        subRange := (1 + Trunc(autoMin / 0.25)) * 0.25;
        autoRange := subRange + 1.30;
      end; {autoRange = total range plus headroom and footroom}

      {subRange is the least multiple of -0.25 that bounds automin}
      {The bottom of mhe bounding box is automin - 0.15 }

  {Draw the bounding box}
    if autoMin = 0.0 then
      YORIGIN := SCREENBOX.Y2
    else
      YORIGIN := SCREENBOX.Y1 + Round((1.15 / autoRange) * BOXHGT);

    XORIGIN := SCREENBOX.X1;
    Rectangle(SCREENBOX.X1, SCREENBOX.Y1, SCREENBOX.X2, SCREENBOX.Y2);
    MoveTo(SCREENBOX.X1, YORIGIN);
    LineTo(SCREENBOX.X2, YORIGIN);

    HR := XORIGIN + 5;
    VT := YORIGIN;
    toly := 0.0;

    while toly <= 1.2 do
      begin {Mark the Vertical Axis from zero to 1.2}
        MoveTo(XORIGIN, VT);
        LineTo(HR, VT);
        tolyString := TalyToString(toly);
        tolyWidth := TextWidth(tolyString);
        PENLEFT := tolyWidth + 15;
        MoveTo(XORIGIN - PENLEFT, VT + PENDOWN);
        OutText(tolyString);
        toly := toly + taly;
        VT := YORIGIN - Round(BOXHGT * (toly / autoRange));
      end; {Mark the vertical axis}

    toly := taly;

    if autoMin <> 0 then
      begin {Mark the vertical axis below the horizontal}
        while toly <= subRange + 0.2 do
          begin
            VT := YORIGIN + Round(BOXHGT * (toly / autoRange));
            MoveTo(XORIGIN, VT);
            LineTo(HR, VT);
            tolyString := TalyToString(-toly);
            tolyWidth := TextWidth(tolyString);
            PENLEFT := tolyWidth + 15;
            MoveTo(XORIGIN - PENLEFT - 3, VT + PENDOWN);
            OutText(tolyString);
            toly := toly + taly;
          end;
      end; {Mark the vertical axis below the horizontal}

  {Mark the Horizontal axis}
    xRange := lags + 1;
    TallyPoints(xRange, taly);
    HSTEP := BOXWDT * taly / xRange;
    MarksAndNumbers(SCREENBOX.X1, SCREENBOX.Y2, taly, 0, xRange, HSTEP, 'xScale');

    SCREENBOX.X0 := XORIGIN;
    SCREENBOX.Y0 := YORIGIN;

  end; {AutoFrame}

 {________________________________________________}

  procedure AutoSpikes (lags: integer;
                  var autoBOX: rBOX;
                  var SCREENBOX: IBOX;
                  rho: shortVector;
                  autoRange: real);

    var
      XORIGIN, YORIGIN, BOXWDT, BOXHGT, HR, VT, i: integer;
      oldSettings: lineSettingsType;

  begin
    XORIGIN := Round(SCREENBOX.X0);
    YORIGIN := Round(SCREENBOX.Y0);
    BOXWDT := SCREENBOX.X2 - SCREENBOX.X1;
    BOXHGT := SCREENBOX.Y2 - SCREENBOX.Y1;

    GetLineSettings(oldSettings);
    SetLineStyle(solidLn, 0, thickWidth);

    for i := 1 to lags do
      begin
        HR := XORIGIN + Round(BOXWDT * i / (lags + 1));
        VT := YORIGIN - Round(BOXHGT * (rho[i] / autoRange));
        MoveTo(HR, YORIGIN);
        LineTo(HR, VT);
      end;

    SetLineStyle(oldSettings.lineStyle, oldSettings.pattern, oldSettings.thickness);

  end; {AutoSpikes}

 {________________________________________________}


end. {SCREEN: Unit 5 of CONCRETE.PAS}

