unit SPECTRUM; {Unit 7 of CONCRETE.PAS}

interface

  uses
    GLOBALS, MATHS, SCREEN, UTILS, POST, LEGENDS, Graph, Dos, Crt;


  procedure Laurent (var mu, gamma: shortVector;
                  q: integer);

  function Spectrum (chi, phi: shortVector;
                  omega: real;
                  var ARMASpec: ARMARecord): real;

  function SDEResponse (omega: real;
                  p: integer;
                  var alpha: shortVector): real;

  procedure SpectrumAndOmega (chi, phi: shortVector;
                  var ARMASpec: ARMARecord;
                  var realBOX: rBOX;
                  var SCREENBOX: IBOX;
                  var dataVector: longVector);

  procedure PlotSpectrum (var SCREENBOX, POSTBOX: IBOX;
                  var ARMASpec: ARMARecord);

  procedure PlotLSDESpectrum (var SCREENBOX, POSTBOX: IBOX;
                  var spectraBOX: rBOX;
                  var LSDESpec, ARMASpec: ARMARecord;
                  var piMultiple: integer;
                  overlayTheARMA: boolean);

implementation

 {_________________________________________________}

  procedure Laurent (var mu, gamma: shortVector;
                  q: integer);

    var
      i, j: integer;

  begin
    for i := 0 to q do
      begin {i}
        gamma[i] := 0.0;
        for j := 0 to q - i do
          gamma[i] := gamma[i] + mu[j] * mu[j + i];
      end; {i}
  end; {Laurent}

 {_________________________________________________}

  function Spectrum (chi, phi: shortVector;
                  omega: real;
                  var ARMASpec: ARMARecord): real;

    var
      j: integer;
      numerator, denominator: extended;

  begin
   with ARMASpec do
    begin
     numerator := phi[0];
     for j := 1 to q do
      numerator := numerator + 2 * phi[j] * Cos(omega * j);
     denominator := chi[0];
     for j := 1 to p do
      denominator := denominator + 2 * chi[j] * Cos(omega * j);
     Spectrum := varEpsilon * (numerator / denominator);
    end;
  end; {Spectrum}

 {_________________________________________________}

  function SDEResponse (omega: real;
                  p: integer;
                  var alpha: shortVector): real;

{This serves for both the numerator and the denominator of the spectrum of the LDSE}

    var
      i, j, k, factor: integer;
      sum, omegaTwo, omegaPower: real;

  begin
    sum := 0.0;
    omegaTwo := omega * omega;
    omegaPower := 1.0;
    factor := -2;
    for i := 0 to p do
      begin{i}
        j := 0;
        k := p - i;
        sum := sum + alpha[p - i] * alpha[p - i] * omegaPower;
        while ((k - j) > 0) and ((k + j) < p) do
          begin
            j := j + 1;
            sum := sum + factor * alpha[k - j] * alpha[k + j] * omegaPower;
            factor := -factor;
          end;
        omegaPower := omegaPower * omegaTwo;
      end; {i}
    SDEResponse := sum;

  end; {SDEResponse}

 {_________________________________________________}

  procedure SpectrumAndOmega (chi, phi: shortVector;
                  var ARMASpec: ARMARecord;
                  var realBOX: rBOX;
                  var SCREENBOX: IBOX;
                  var dataVector: longVector);

    var
      i, BOXWDT: integer;
      omega, increment: real;

  begin {SpectrumAndOmega}

    BOXWDT := SCREENBOX.X2 - SCREENBOX.X1;

    realBOX.xMax := pi; {the maximum value of x in the data vector}
    realBOX.xMin := 0.0; {the minmum value  of x in the data vector}
    realBOX.yMax := 0.0; {the maximum value of y in the Data vector}
    realBOX.yMin := 0.0; {the minimum value of y in the Data vector}
    omega := 0.0;
    increment := pi / BOXWDT;

    for i := 0 to BOXWDT do
      begin {i}
        dataVector[i] := Spectrum(chi, phi, omega, ARMASpec);
        if dataVector[i] > realBOX.yMax then
          realBOX.yMax := dataVector[i];
        omega := omega + increment;
      end; {i}

  end; {SpectrumAndOmega}

{_________________________________________________}

  procedure PlotSpectrum (var SCREENBOX, POSTBOX: IBOX;
                  var ARMASpec: ARMARecord);
    var
      phi, chi: shortVector;
      spectraVec: longVector;
      spectraBOX: rBOX;
      postFileName, legend: string;
      F: text;

  begin
    ClrScr;

    Laurent(ARMASpec.beta, phi, ARMASpec.q);
    Laurent(ARMASpec.alpha, chi, ARMASpec.p);

    SpectrumAndOmega(chi, phi, ARMASpec, spectraBOX, SCREENBOX, spectraVec);
    GetGraphics(graphicsOn, SCREENBOX, currentMode);
    ScreenFrameLite(spectraBOX, SCREENBOX);
    ZeroPiAxis(SCREENBOX);
    ScreenPlotMod(spectraBOX, SCREENBOX, spectraVec);
    legend := ARMASpectLegend(ARMASpec);
    MultiCaption(legend);

    CallForReturn;
    Readln;
    CloseGraph;
    RestoreCrtMode;
    ClrScr;

    if PSPict then
      begin {make a PostScript File}
        Write('    Name the PostScript File = ');
        Readln(postFileName);
          begin {postFileName}
            Assign(F, postFileName);
            ReWrite(F);
            PostHeader(F, POSTBOX, TeXFormat);
            PostFrameLite(F, spectraBOX, POSTBOX);
            PostZeroPiAxis(F, POSTBOX);
            PostPlotMod(F, spectraBOX, POSTBOX, spectraVec);
            PostFooter(F, POSTBOX, legend, TexFormat);
            Close(F);
          end; {postFileName}

      end;{make a PostScript File}

  end; {PlotSpectrum}

{_________________________________________________}

  procedure LSDESpectalOrinates (var alpha, beta: shortVector;
                  p, q: integer;
                  var spectraBOX: rBOX;
                  var SCREENBOX: IBOX;
                  var spectraVec: longVector);

    var
      i, BOXWDT: integer;
      omega, increment, numerator, denominator: real;

  begin
    BOXWDT := SCREENBOX.X2 - SCREENBOX.X1;

    spectraBOX.xMin := 0.0; {the minmum value  of x in the data vector}
    spectraBOX.yMax := 0.0; {the maximum value of y in the Data vector}
    spectraBOX.yMin := 0.0; {the minimum value of y in the Data vector}
    omega := 0.0;
    increment := spectraBOX.xMax / BOXWDT;

    for i := 0 to BOXWDT do
      begin {i}
        numerator := SDEResponse(omega, q, beta);
        denominator := SDEResponse(omega, p, alpha);
        spectraVec[i] := numerator / denominator;
        if spectraVec[i] > spectraBOX.yMax then
          spectraBOX.yMax := spectraVec[i];
        omega := omega + increment;
      end;  {i}

  end; {SDESpectalOrinates}

 {_________________________________________________}

  procedure PlotLSDESpectrum (var SCREENBOX, POSTBOX: IBOX;
                  var spectraBOX: rBOX;
                  var LSDESpec, ARMASpec: ARMARecord;
                  var piMultiple: integer;
                  overlayTheARMA: boolean);

    var
      phi, chi: shortVector;
      spectraVec, ARMAVec: longVector;
      postFileName, legend: string;
      F: text;
      SUBSCREEN, SUBPOST: IBOX;
      subBOX: rBOX;
      BOXWIDTH: integer;
      oldSettings: lineSettingsType;

  begin

    with LSDESpec do
      LSDESpectalOrinates(alpha, beta, p, q, spectraBOX, SCREENBOX, spectraVec);

    if (not overlayTheARMA) then
      begin {if}
        GetGraphics(graphicsOn, SCREENBOX, currentMode);
        ScreenFrameLite(spectraBOX, SCREENBOX);
        PolyPiAxis(SCREENBOX, spectraBOX);
        ScreenPlotMod(spectraBOX, SCREENBOX, spectraVec);
        legend := LSDESpectLegend(ARMASpec);
        MultiCaption(legend);
      end; {if}


    if overlayTheARMA then
      begin {if}
        GetGraphics(graphicsOn, SCREENBOX, currentMode);

{fix the limits of the plot of the ARMA spectrum}
        SUBSCREEN := SCREENBOX;
        BOXWIDTH := SCREENBOX.X2 - SCREENBOX.X1;
        BOXWIDTH := Round(BOXWIDTH / piMultiple);
        SUBSCREEN.X2 := SCREENBOX.X1 + BOXWIDTH;

{Find the ordinates of the ARMA over theSUBSCREEN}
        Laurent(ARMASpec.beta, phi, ARMASpec.q);
        Laurent(ARMASpec.alpha, chi, ARMASpec.p);
        SpectrumAndOmega(chi, phi, ARMASpec, subBOX, SUBSCREEN, ARMAVec);

{Erect the frame for the overplot}
{subBOX records the yMax value for the ARMA spectrum}
        ScreenFrameLite(subBOX, SCREENBOX);
        PolyPiAxis(SCREENBOX, spectraBOX);
        MoveTo(SUBSCREEN.X2, SUBSCREEN.Y1);
        LineTo(SUBSCREEN.X2, SUBSCREEN.Y2);

{Plot the ARMASpectrum on a sub-interval of the Spectral range of the LSDE}
        SetColor(red);
        GetLineSettings(oldSettings);
        SetLineStyle(solidLn, 0, thickWidth);
        ScreenPlotMod(subBOX, SUBSCREEN, ARMAVec);
        SetLineStyle(oldSettings.lineStyle, oldSettings.pattern, oldSettings.thickness);

{Plot the LSDESpectrum over the entire spectral range}
        SetColor(yellow);
        ScreenPlotMod(subBOX, SCREENBOX, spectraVec);

        legend := OverLaySpectLegend (LSDESpec, ARMASpec);
        MultiCaption(legend);
      end; {if}

    CallForReturn;
    Readln;
    CloseGraph;
    RestoreCrtMode;
    ClrScr;

    if PSPict then
      begin {make a PostScript File}
        legend := ' The spectrum of the LSDE';
        Write('    Name the PostScript File = ');
        Readln(postFileName);
        if postFileName <> '' then
          begin {postFileName}
            Assign(F, postFileName);
            ReWrite(F);
            PostHeader(F, POSTBOX, TeXFormat);

            if (not overlayTheARMA) then
              begin {if}
                PostFrameLite(F, spectraBOX, POSTBOX);
                PostPolyPiAxis(F, POSTBOX, spectraBOX);
                PostPlotMod(F, spectraBOX, POSTBOX, spectraVec);
              end; {if}

            if overlayTheARMA then
              begin {if}
                SUBPOST := POSTBOX;
                SUBPOST.X2 := Round(POSTBOX.X2 / piMultiple);
                PostFrameLite(F, spectraBOX, POSTBOX);
                PostPolyPiAxis(F, POSTBOX, spectraBOX);

                Writeln(F, 'newpath');
                Writeln(F, SUBPOST.X2 : 2, '  ', SUBPOST.Y1 : 2, '  moveto');
                Writeln(F, SUBPOST.X2 : 2, '  ', SUBPOST.Y2 : 3, '  lineto');
                Writeln(F, 'stroke');

                PostPlotMod(F, subBOX, SUBPOST, ARMAVec);
                PostPlotMod(F, subBOX, POSTBOX, spectraVec);
                Writeln(F, 'stroke');
                legend := 'The spectrum of the LSDE and the ARMA';
              end; {if}

            PostFooter(F, POSTBOX, legend, TexFormat);
            Close(F);
          end; {postFileName}
      end;{if PSPict}

  end; {PlotLSDESpectrum}

 {_________________________________________________}

 {SPECTRUM; Unit 7 of CONCRETE.PAS}

end. {SPECTRUM: Unit 7 of CONCRETE.PAS }

